/*
 *
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved. 
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 * 
 */
import javax.xml.registry.*; 
import javax.xml.registry.infomodel.*; 
import java.net.*;
import java.util.*;

/**
 * The JAXRQuery class consists of a main method, an 
 * executeQuery method, and some private helper methods. It
 * searches a registry for information about organizations whose 
 * names contain a user-supplied string.
 * 
 * To run this program, use the command 
 * 
 *     ant -Dquery-string=<value> run-query
 *
 * after starting Tomcat.
 */
public class JAXRQuery {

    public JAXRQuery() {}

    public static void main(String[] args) {

        if (args.length != 1) {
            System.out.println("Usage: java " +
                "JAXRQuery <query-string>");
            System.exit(1);
        }
        String queryString = new String(args[0]);
        System.out.println("Query string is " + queryString);
        
        JAXRQuery jq = new JAXRQuery();
        jq.executeQuery(queryString);
    }
    
    /**
     * Searches for organizations containing a string and 
     * displays data about them.
     *
     * @param qString	the string argument
     */
    public void executeQuery(String qString) {
        Connection connection = null;

        // Define connection configuration properties 
        // To query, you need only the query URL
        Properties props = new Properties();
        props.setProperty("javax.xml.registry.queryManagerURL", 
   "http://www-3.ibm.com/services/uddi/testregistry/inquiryapi");
        props.setProperty("javax.xml.registry.factoryClass", 
            "com.sun.xml.registry.uddi.ConnectionFactoryImpl");

        try {
            // Create the connection, passing it the 
            // configuration properties
            ConnectionFactory factory = 
                ConnectionFactory.newInstance();
            factory.setProperties(props);
            connection = factory.createConnection();

            // Get registry service and query manager
            RegistryService rs = connection.getRegistryService();
            BusinessQueryManager bqm = 
                rs.getBusinessQueryManager();
            System.out.println("Got registry service and " +
                "query manager");

            // Define find qualifiers and name patterns
            Collection findQualifiers = new ArrayList();
            findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
            Collection namePatterns = new ArrayList();
            namePatterns.add("%" + qString + "%");

            // Find using the name
            BulkResponse response = 
                bqm.findOrganizations(findQualifiers, 
                    namePatterns, null, null, null, null);
            Collection orgs = response.getCollection();

            // Display information about the organizations found
            Iterator orgIter = orgs.iterator();
            while (orgIter.hasNext()) {
                Organization org = 
                    (Organization) orgIter.next();
                System.out.println("Org name: " + getName(org));
                System.out.println("Org description: " + 
                    getDescription(org));
                System.out.println("Org key id: " + getKey(org));

                // Display primary contact information
                User pc = org.getPrimaryContact();
                if (pc != null) {
                    PersonName pcName = pc.getPersonName();
                    System.out.println(" Contact name: " + 
                        pcName.getFullName());
                    Collection phNums = 
                        pc.getTelephoneNumbers(pc.getType());
                    Iterator phIter = phNums.iterator();
                    while (phIter.hasNext()) {
                        TelephoneNumber num = 
                            (TelephoneNumber) phIter.next();
                        System.out.println("  Phone number: " + 
                            num.getNumber());
                    }
                    Collection eAddrs = pc.getEmailAddresses();
                    Iterator eaIter = eAddrs.iterator();
                    while (phIter.hasNext()) {
                        System.out.println("  Email Address: " + 
                            (EmailAddress) eaIter.next());
                    }
                }

                // Display service and binding information
                Collection services = org.getServices();
                Iterator svcIter = services.iterator();
                while (svcIter.hasNext()) {
                    Service svc = (Service) svcIter.next();
                    System.out.println(" Service name: " + 
                        getName(svc));
                    System.out.println(" Service description: " +
                        getDescription(svc));
                    Collection serviceBindings = 
                        svc.getServiceBindings();
                    Iterator sbIter = serviceBindings.iterator();
                    while (sbIter.hasNext()) {
                        ServiceBinding sb = 
                            (ServiceBinding) sbIter.next();
                        System.out.println("  Binding " +
                            "Description: " + 
                            getDescription(sb));
                        System.out.println("  Access URI: " + 
                            sb.getAccessURI());
                    }
                }
                // Print spacer between organizations
                System.out.println(" --- "); 
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally  {
            // At end, close connection to registry
            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {}
            }
        }
    }

    /**
     * Returns the name value for a registry object.  Returns
     * an empty string if the value is null.
     *
     * @param ro	a RegistryObject
     * @return		the String value
     */
    private String getName(RegistryObject ro) throws JAXRException {
        try {
            return ro.getName().getValue();
        } catch (NullPointerException npe) {
            return "";
        }
    }
    
    /**
     * Returns the description value for a registry object.
     * Returns an empty string if the value is null.
     *
     * @param ro	a RegistryObject
     * @return		the String value
     */
    private String getDescription(RegistryObject ro) throws JAXRException {
        try {
            return ro.getDescription().getValue();
        } catch (NullPointerException npe) {
            return "";
        }
    }
    
    /**
     * Returns the key id value for a registry object.
     * Returns an empty string if the value is null.
     *
     * @param ro	a RegistryObject
     * @return		the String value
     */
    private String getKey(RegistryObject ro) throws JAXRException {
        try {
            return ro.getKey().getId();
        } catch (NullPointerException npe) {
            return "";
        }
    }
}